@@ -1,3 +1,9 @@
+/*
+ * SPDX-FileCopyrightText: 2019-2023 Mattia Basaglia <dev@dragon.best>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #pragma once
 
 #include <memory>
@@ -18,24 +24,6 @@ namespace glaxnimate::model {
 
 namespace glaxnimate::io::svg {
 
-class SvgParseError : public std::exception
-{
-public:
-    QString formatted(const QString& filename) const
-    {
-        return QString("%1:%2:%3: SVG Parse Error: %4")
-            .arg(filename)
-            .arg(line)
-            .arg(column)
-            .arg(message)
-        ;
-    }
-
-    QString message;
-    int line = -1;
-    int column = -1;
-};
-
 class SvgParser
 {
 public:
@@ -56,7 +44,9 @@ public:
         model::Document* document,
         const std::function<void(const QString&)>& on_warning = {},
         ImportExport* io = nullptr,
-        QSize forced_size = {}
+        QSize forced_size = {},
+        model::FrameTime default_time = 180,
+        QDir default_asset_path = {}
     );
     ~SvgParser();
 
