@@ -1,9 +1,144 @@
+/*
+ * SPDX-FileCopyrightText: 2019-2023 Mattia Basaglia <dev@dragon.best>
+ *
+ * SPDX-License-Identifier: GPL-3.0-or-later
+ */
+
 #include "animatable.hpp"
 
 #include "command/animation_commands.hpp"
 #include "model/object.hpp"
 #include "math/bezier/segment.hpp"
 
+
+std::vector<std::unique_ptr<glaxnimate::model::KeyframeBase>> glaxnimate::model::KeyframeBase::split(const KeyframeBase* other, std::vector<qreal> splits) const
+{
+    std::vector<std::unique_ptr<KeyframeBase>> kfs;
+    if ( transition().hold() )
+    {
+        kfs.push_back(clone());
+        kfs.push_back(other->clone());
+        return kfs;
+    }
+
+    auto splitter = this->splitter(other);
+
+    kfs.reserve(splits.size()+2);
+    qreal prev_split = 0;
+    const KeyframeBase* to_split = this;
+    std::unique_ptr<KeyframeBase> split_right;
+    QPointF old_p;
+    for ( qreal split : splits )
+    {
+        // Skip zeros
+        if ( qFuzzyIsNull(split) )
+            continue;
+
+        qreal split_ratio = (split - prev_split) / (1 - prev_split);
+        prev_split = split;
+        auto transitions = to_split->transition().split_t(split_ratio);
+        // split_ratio is t
+        // p.x() is time lerp
+        // p.y() is value lerp
+        QPointF p = transition().bezier().solve(split);
+        splitter->step(p);
+        auto split_left = splitter->left(old_p);
+        split_left->set_transition(transitions.first);
+        old_p = p;
+        split_right = splitter->right(p);
+        split_right->set_transition(transitions.second);
+        to_split = split_right.get();
+        kfs.push_back(std::move(split_left));
+    }
+    kfs.push_back(std::move(split_right));
+    kfs.push_back(splitter->last());
+    kfs.back()->set_transition(other->transition());
+
+    return kfs;
+}
+
+
+class glaxnimate::model::Keyframe<QPointF>::PointKeyframeSplitter : public KeyframeSplitter
+{
+public:
+    const Keyframe* self;
+    const Keyframe* other;
+    math::bezier::CubicBezierSolver<QPointF> solver;
+    math::bezier::LengthData len;
+    QPointF tan_in;
+    math::bezier::Point point_before;
+    math::bezier::Point point_mid;
+    qreal prev_split = 0;
+    bool linear;
+
+    PointKeyframeSplitter(const Keyframe<QPointF>* self, const Keyframe<QPointF>* other)
+        : self(self),
+          other(other),
+          solver(self->bezier_solver(*other)),
+          len(solver, 20),
+          tan_in(self->point_.tan_in),
+          linear(self->is_linear())
+    {
+    }
+
+    void step(const QPointF& p) override
+    {
+        if ( linear )
+            return;
+
+        // TODO: would need the ability to access other keyframes to properly handle values outside [0, 1]
+        qreal split = math::bound(0., p.y(), 1.);
+        auto beziers = solver.split((split - prev_split) / (1 - prev_split));
+        prev_split = split;
+        solver = beziers.second;
+        point_before = math::bezier::Point (beziers.first[0], tan_in, beziers.first[1]);
+        point_mid = math::bezier::Point (beziers.first[3], beziers.first[2], beziers.second[1]);
+        tan_in = beziers.second[2];
+    }
+
+    std::unique_ptr<KeyframeBase> left(const QPointF& p) const override
+    {
+        if ( linear )
+        {
+            return std::make_unique<Keyframe>(
+                math::lerp(self->time(), other->time(), p.x()),
+                math::lerp(self->get(), other->get(), p.y())
+            );
+        }
+
+        return std::make_unique<Keyframe>(math::lerp(self->time(), other->time(), p.x()), point_before);
+    }
+
+    std::unique_ptr<KeyframeBase> right(const QPointF& p) const override
+    {
+        if ( linear )
+        {
+            return std::make_unique<Keyframe>(
+                math::lerp(self->time(), other->time(), p.x()),
+                math::lerp(self->get(), other->get(), p.y())
+            );
+        }
+
+        return std::make_unique<Keyframe>(math::lerp(self->time(), other->time(), p.x()), point_mid);
+    }
+
+    std::unique_ptr<KeyframeBase> last() const override
+    {
+        if ( linear )
+            return other->clone();
+
+        math::bezier::Point point_after = other->point();
+        point_after.tan_in = tan_in;
+        return std::make_unique<Keyframe>(other->time(), point_after);
+    }
+
+};
+
+std::unique_ptr<glaxnimate::model::KeyframeBase::KeyframeSplitter> glaxnimate::model::Keyframe<QPointF>::splitter(const KeyframeBase* other) const
+{
+    return std::make_unique<PointKeyframeSplitter>(this, static_cast<const Keyframe<QPointF>*>(other));
+}
+
 bool glaxnimate::model::AnimatableBase::assign_from(const model::BaseProperty* prop)
 {
     if ( prop->traits().flags != traits().flags || prop->traits().type != traits().type )
@@ -177,8 +312,7 @@ void glaxnimate::model::detail::AnimatedPropertyPosition::split_segment(int inde
 
 bool glaxnimate::model::detail::AnimatedPropertyPosition::set_bezier(math::bezier::Bezier bezier)
 {
-    if ( bezier.closed() && !bezier.empty() && !math::fuzzy_compare(bezier[0].pos, bezier.back().pos) )
-        bezier.push_back(bezier[0]);
+    bezier.add_close_point();
 
     // TODO if sizes don't match, re-arrange keyframes based on
     // how far keyframes are in the bezier
@@ -319,3 +453,34 @@ void glaxnimate::model::detail::AnimatedPropertyPosition::add_smooth_keyframe_un
 
     object()->document()->push_command(parent.release());
 }
+
+std::optional<QPointF> glaxnimate::model::detail::AnimatedPropertyPosition::derivative_at(glaxnimate::model::FrameTime time) const
+{
+    int count = keyframe_count();
+    if ( count < 2 )
+        return {};
+
+    int index_before = keyframe_index(time);
+    const keyframe_type* kf_before = keyframe(index_before);
+    const keyframe_type* kf_after = nullptr;
+    qreal factor = 1;
+
+    if ( index_before == count - 1 )
+    {
+        kf_after = kf_before;
+        kf_before = keyframe(index_before - 1);
+    }
+    else
+    {
+        kf_after = keyframe(index_before + 1);
+        factor = math::unlerp(kf_before->time(), kf_after->time(), time);
+    }
+
+    const math::bezier::Point& point_before = kf_before->point();
+    const math::bezier::Point& point_after = kf_after->point();
+    math::bezier::CubicBezierSolver solver(point_before.pos, point_before.tan_out, point_after.tan_in, point_after.pos);
+    return QPointF(
+        solver.derivative(factor, 0),
+        solver.derivative(factor, 1)
+    );
+}
